/*
 * bits64/stddef.h
 */

#ifndef _BITSIZE_STDDEF_H
#define _BITSIZE_STDDEF_H

#define _SIZE_T
typedef unsigned long size_t;
#define _PTRDIFF_T
typedef signed long   ptrdiff_t;

#endif /* _BITSIZE_STDDEF_H */
