/*
 * machine/asm.h
 */

#ifndef _MACHINE_ASM_H
#define _MACHINE_ASM_H

/* Standard aliases for Alpha register names */

#define v0	$0
#define t0	$1
#define t1	$2
#define t2	$3
#define t3	$4
#define t4	$5
#define t5	$6
#define t6	$7
#define t7	$8
#define s0	$9
#define s1	$10
#define s2	$11
#define s3	$12
#define s4	$13
#define s5	$14
#define fp	$15
#define a0	$16
#define a1	$17
#define a2	$18
#define a3	$19
#define a4	$20
#define a5	$21
#define t8	$22
#define t9	$23
#define t10	$24
#define t11	$25
#define ra	$26
#define t12	$27		/* t12 and pv are both used for $27 */
#define pv	$27		/* t12 and pv are both used for $27 */
#define at	$28
#define gp	$29
#define sp	$30
#define zero	$31

#endif /* _MACHINE_ASM_H */
