/*
 * dummyportmap.h
 *
 * Functions for the portmap spoofer
 */

#ifndef KINIT_DUMMYPORTMAP_H
#define KINIT_DUMMYPORTMAP_H

#include <unistd.h>
pid_t start_dummy_portmap(const char *file);

#endif /* KINIT_DUMMYPORTMAP_H */
